/**
 * @file	cgic.h
 *
 * @brief	This library is intended to make the CGI C programmer's life easier, using a set of
 * functions to read fields from HTML forms
 *
 * @author	BlackLight	blacklight86@gmail.com
 * @licence	GPL
 */

#ifndef _CGIC_H
#define _CGIC_H	1

/**
 * @brief		This functions sets the 'Expire' field in the HTTP header, it means
 * 			when the information in the header becomes invalid
 * @param		ex		A struct tm-pointer, with the fields pointing to the expire-time
 */
void expires (struct tm *ex);

/**
 * @brief      This functions sets the 'Last-Modified' field in the HTTP header
 * @param      last        A struct tm-pointer, with the fields pointing to the last-modified-time
 */
void last_modified (struct tm *last);

/**
 * @brief		This function sets a cookie on the client, setting up the 'Set-Cookie'
 * 			field in the HTTP header
 * @param		name		The name of the cookie
 * @param		content	Cookie's content
 */
void set_cookie (char *name, char *content);

/*
 * @brief		This function sets manually the content of the HTTP header, or of one of its fields
 * @param		head		Content of the header
 */
void http_header(char *head);

/**
 * @brief		This function sets up the 'Location' field in the HTTP header, used for redirects
 * @param		url		The URL of the resource to redirect
 */
void location (char *url);

/**
 * @brief		This function returns the server name
 */
char* server_name();

/**
 * @brief		This function returns the type of server installed and its version
 */
char* server_software();

/**
 * @brief		This function returns the method invoked by HTTP client (GET, POST, HEAD...)
 */
char* request_method();

/**
 * @brief		This function self-references the application
 */
char* self();

/**
 * @brief		This function returns the name of the remote client or, in case this should't
 * 			be set, its address
 */
char* remote_host();

/**
 * @brief		This function returns the address of the remote client
 */
char* remote_addr();

/**
 * @brief		This function returns the 'User-Agent' field of the remote client
 */
char* user_agent();

/**
 * @brief		This function sets an arbitrary content for 'Host' field in HTTP header
 * @param		name		The name to be set
 */
void set_server_name(char *name);

/**
 * @brief		This function sets an arbitrary content for 'Server' field in HTTP header
 * @param		soft		The version to be set
 */
void set_server_software(char *soft);

/**
 * @brief		This function gets a string in URL or query-string format and converts it into ASCII format
 * @param		src		The source string in URL-format
 * @param		dest		The buffer where the ASCII output is going to be saved
 * @param		len		The length of the destination buffer
 */
void unencode(char *src, char *dest, int len);

/**
 * @brief		This function gets the query string, if set, in case the client sent a GET request
 * 			from a HTML form
 */
char* get_query();

/**
 * @brief		This function gets the query string, if set, in case the client sent a POST request
 */
char* post_query();

/**
 * @brief		This function parses the input inserted by a user, converting < and > tags into the
 * 			right HTML entities, ' and " into their escaped formats, to avoid XSS and SQL injection
 * 			vulnerabilities in CGI apps
 * @param		input		String to be parsed
 */
char* parse_input(char *input);

/**
 * @brief This function gets the content of a POST or GET form's field
 * @param	query	A string where is saved the content of POST query
 * 				(you can get it with get_post_query() or get_get_query() methods
 * @param	field	The name of the field
 * @example
 * 				<! This is the HTML code >
 * 				<form method="POST" action="/cgi-bin/post.cgi">
 * 					Username: <input name="user">
 * 					<input type="submit" value="Login">
 * 				</form>
 *
 *				// This is the content of file /cgi-bin/post.cgi.c
 *				char *user,*query;
 *
 *				if ( (query=post_query()) == NULL )  {
 *					printf ("Error - invalid form\n");
 *					return;
 *				}
 *
 *				if ( (user=get_field(query,"user")) == NULL )
 *					printf ("Error - invalid field\n");
 *				else
 *					printf ("Welcome %s!\n", user);
 */
char* get_field(char *query, char *field);

/**
 * @brief		This function gets the content of a cookie on the client
 * @param		cookie		Name of the cookie to get
 */
char* cookie(char *cookie);
#endif

